/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* * [VERSION 1.5]                                                                                                   * */
/* *                                                                                                                 * */
/* * INSTRUCTIONS FOR USE:                                                                                           * */
/* *                                                                                                                 * */
/* * This file defines some example font variations to demonstrate the many possible ways that characters with       * */
/* * different border widths can be combined in interesting ways. Upon original creation, the characters occupy the  * */
/* * space from z = 0 to z = 1. The finished "stylized" characters produced by the Stylize () macro in this file     * */
/* * occupy the space from z = -TextDepth to z = 0.                                                                  * */
/* *                                                                                                                 * */
/* * The file "Sample_FontVariations.pov" demonstrates each of the styles defined in this file. Render the file to   * */
/* * see what the pre-defined sample font variations look like. Note that you'll need to set the variable            * */
/* * [StartFontVariationNumber] as explained in the comments and re-render the file to see all the available font    * */
/* * variations.                                                                                                     * */
/* *                                                                                                                 * */
/* * You'll need to define at least one "Font Style" prior to using these sample font variations. See the            * */
/* * instructions for the SetFontStyle () macro in the "BorderChars_Macros.inc" file.                                * */
/* *                                                                                                                 * */
/* * Sample font variation #4 also requires two global colors, named [ExtraParameter_Color0] and                     * */
/* * [ExtraParameter_Color1].                                                                                        * */
/* *                                                                                                                 * */
/* * I realize that assigning textures to the characters will make it more difficult to integrate them with other    * */
/* * objects in your scene. This is a trade-off that will require a little more work on your part if you wish to do  * */
/* * this.                                                                                                           * */
/* *                                                                                                                 * */
/* * The easiest way to use these sample font variations in your own scenes is to copy and paste the variable names  * */
/* * and example values from "Sample_FontVariations.pov" into your scene then adjust the values as necessary. For    * */
/* * example, if you wanted to use font variation #15, you would scroll down until you find the line "#case (15)"    * */
/* * then cut and past the following code:                                                                           * */
/* *                                                                                                                 * */
/* *    #local FontVariationNumber = 15;                                                                             * */
/* *    #local TextDepth = .08;                                                                                      * */
/* *    #local InvisibleBorderWidth = 0;                                                                             * */
/* *    #local BorderWidth = .03;                                                                                    * */
/* *    #local StripeWidth = .02;                                                                                    * */
/* *    #local StripeDepth = .02;                                                                                    * */
/* *    #local InnerHeight = .03;                                                                                    * */
/* *    SetFontStyle (FontStyleIndex, FontVariationNumber, TextDepth, InvisibleBorderWidth, BorderWidth,             * */ 
/* *       StripeWidth, StripeDepth, InnerHeight, -999, TextColor_Outer, TextColor_Inner, TextColor_Back)            * */
/* *                                                                                                                 * */
/* * In this example, the variable names "InvisibleBorderWidth," "BorderWidth," etc. represent the parameters "P1,"  * */
/* * "P2," etc. and the value -999 is used for "P6."                                                                 * */
/* *                                                                                                                 * */
/* * There are two variables used within these samples that require special explanation:                             * */
/* *                                                                                                                 * */
/* *    [TextScale_Final] Since the character objects will be scaled along the x and y axes by [TextScale_Final],    * */
/* *    this could also change the border widths. Therefore, widths are divided by the value of [TextScale_Final]    * */
/* *    prior to character creation so that the final border width after scaling will be correct.                    * */
/* *                                                                                                                 * */
/* *    [InvisibleBorderWidth] This variable can be used to decrease the stroke width of some of the font            * */
/* *    variations. Good values are 0 (no reduction in stroke width) to around .05. This feature does not work       * */
/* *    perfectly, and does not work at all with the SCALED micro-justification method.                              * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * CREATE THE VARIOUS FONT VARIATIONS                                                                              * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#macro Stylize (FontStyleIndex, TextScale_Final)

   #declare FontStyleRecord_Valid [FontStyleIndex] = true;

   #local FontVariationNumber = FontStyleRecord_FontVariationNumber [FontStyleIndex];
   #local TextDepth = FontStyleRecord_TextDepth [FontStyleIndex];
   #local P1 = FontStyleRecord_P1 [FontStyleIndex];
   #local P2 = FontStyleRecord_P2 [FontStyleIndex];
   #local P3 = FontStyleRecord_P3 [FontStyleIndex];
   #local P4 = FontStyleRecord_P4 [FontStyleIndex];
   #local P5 = FontStyleRecord_P5 [FontStyleIndex];
   #local P6 = FontStyleRecord_P6 [FontStyleIndex];
   #local TextColor_Outer = texture {FontStyleRecord_TextColor_Outer [FontStyleIndex]}
   #local TextColor_Inner = texture {FontStyleRecord_TextColor_Inner [FontStyleIndex]}
   #local TextColor_Back = texture {FontStyleRecord_TextColor_Back [FontStyleIndex]}

   #for (I, 0, 81)
      #switch (FontVariationNumber)
         #case (0) //Plain
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            MakeCharacter (I, InvisibleBorderWidth)
            #declare Character = object {
               intersection {
                  object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               #local S = 1 / (1 - InvisibleBorderWidth * 2);
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (1) //Raised center (use when border and center are the same height)
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local BorderHeight = P3 / 1.1;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character texture {TextColor_Outer}}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {Character texture {TextColor_Inner}}
            #if (BorderHeight = 0)
               #local Character_Outer = object {
                  difference {
                     object {Character_Outer}
                     object {Character_Inner translate -.1 * z}
                  } //difference
               } //object
            #end //#if
            #declare Character = object {
               union {
                  object {Character_Outer}
                  object {Character_Inner translate -BorderHeight * z}
               } //union
            } //object
            #declare Character = object {
               intersection {
                  object {Character scale <1, 1, 1.1>}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (2) //Lowered center
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local BorderDepth = P3;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
            } //object
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {
               intersection {
                  object {Character scale <1, 1, 1.1>}
                  plane {z, 1}
               } //intersection
               texture {TextColor_Inner}
            } //object
            #declare Character = object {
               difference {
                  object {Character_Outer}
                  object {Character_Inner translate (-1 + BorderDepth) * z}
               } //difference
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (3) //Striped
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local StripeWidth = P3 / TextScale_Final;
            #local StripeHeight = P4;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local StripeWidth = StripeWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
            MakeCharacter (I, BorderWidth + StripeWidth + InvisibleBorderWidth)
            #local Character_Inner2 = object {Character scale <1, 1, 1.2>}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {
               difference {
                  object {Character}
                  object {Character_Inner2 translate -.1 * z}
               } //difference
               texture {TextColor_Inner}
            } //object
            #declare Character = object {
               intersection {
                  union {
                     difference {
                        object {Character_Outer}
                        object {Character_Inner translate -.1 * z}
                     } //difference
                     object {Character_Inner translate -StripeHeight * z}
                  } //union
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (4) //Color gradient
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local StripeWidth = P3 / TextScale_Final;
            #local NSteps = P4;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local StripeWidth = StripeWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
            #local StripeWidthIncrement = StripeWidth / NSteps;
            #for (J, 0, NSteps)
               #local ColorRatio = J / (NSteps - 1);
               #local CurColor = ExtraParameter_Color1 * ColorRatio + ExtraParameter_Color0 * (1 - ColorRatio);
               #local CurBorderWidth = BorderWidth + J * StripeWidthIncrement;
               MakeCharacter (I, CurBorderWidth + InvisibleBorderWidth)
               #if (J < NSteps)
                  #local CurTexture = texture {pigment {color CurColor}}
               #else
                  #local CurTexture = texture {TextColor_Outer}
               #end //#if
               #local Character_Inner = object {Character scale <1, 1, 1.3> texture {CurTexture}}
               #local Character_Outer = object {
                  union {
                     difference {
                        object {Character_Outer}
                        object {Character_Inner translate -.1 * z}
                     } //difference
                     object {Character_Inner}
                  } //union
               } //object
            #end //#for
            #declare Character = object {
               intersection {
                  object {Character_Outer}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (5) //Hollow stripe
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local StripeWidth = P3 / TextScale_Final;
            #local StripeDepth = P4;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local StripeWidth = StripeWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character scale <1, 1, 1.1>}
            MakeCharacter (I, BorderWidth + StripeWidth + InvisibleBorderWidth)
            #local Character_Inner2 = object {Character scale <1, 1, 1.2>}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {
               difference {
                  object {Character}
                  object {Character_Inner2 translate -.1 * z}
               } //difference
            } //object
            #declare Character = object {
               intersection {
                  difference {
                     object {Character_Outer}
                     object {Character_Inner translate (-1 + StripeDepth) * z}
                     texture {TextColor_Outer}
                  } //difference
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (6) //Hollow inside
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
            } //object
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {Character scale <1, 1, 1.2> texture {TextColor_Outer}}
            #declare Character = object {
               difference {
                  object {Character_Outer}
                  object {Character_Inner translate -.1 * z}
               } //difference
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (7) //Angled bevel
            #local BorderHeight = P1;
            #local NSteps = P2;
            MakeCharacter (I, 0)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.2> translate -.1 * z texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
                  plane {z, 0 inverse texture {TextColor_Inner}}
               } //intersection
            } //object
            #local Character_Inner = object {
               intersection {
                  union {
                     #local Increment = BorderHeight / NSteps;
                     #for (J, 1, NSteps)
                        MakeCharacter (I, Increment * J)
                        #if (J = NSteps)
                           #declare Character = object {
                              intersection {
                                 object {Character translate -.1 * z}
                                 plane {z, 0 inverse texture {TextColor_Outer}}
                              } //intersection
                           } //object
                        #end //#if
                        object {Character translate (-Increment * J) * z}
                     #end //#for
                  } //union
                  plane {z, 0}
               } //intersection
               texture {TextColor_Inner}
            } //object
            #declare Character = object {
               union {
                  object {Character_Outer}
                  object {Character_Inner}
               } //union
               translate -TextDepth * z //Back is at z = 0
            } //object
         #break //#case
         #case (8) //Rounded bevel
            #local BorderHeight = P1;
            #local NSteps = P2;
            #local R = BorderHeight;
            MakeCharacter (I, 0)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.2> translate -.1 * z texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
                  plane {z, 0 inverse texture {TextColor_Inner}}
               } //intersection
            } //object
            #local Character_Inner = object {
               intersection {
                  union {
                     #local Increment = 90 / NSteps;
                     #for (J, 1, NSteps)
                        #local CurAngle = J * Increment;
                        #local CurBorderWidth = R * (1 + cos (radians (CurAngle + 180)));
                        #local CurBorderHeight = R * sin (radians (CurAngle));
                        MakeCharacter (I, CurBorderWidth)
                        #if (J = NSteps)
                           #declare Character = object {
                              intersection {
                                 object {Character translate -.1 * z}
                                 plane {z, 0 inverse texture {TextColor_Outer}}
                              } //intersection
                           } //object
                        #end //#if
                        object {Character translate -CurBorderHeight * z}
                     #end //#for
                  } //union
                  plane {z, 0}
               } //intersection
               texture {TextColor_Inner}
            } //object
            #declare Character = object {
               union {
                  object {Character_Outer}
                  object {Character_Inner}
               } //union
               translate -TextDepth * z //Back is at z = 0
            } //object
         #break //#case
         #case (9) //Solid hourglass (convex)
            #local EffectWidth = P1;
            #local NSteps = P2;
            #local TempCharacter = object {
               union {
                  #local SegmentLength = TextDepth / NSteps;
                  #local DeltaTheta = 180 / (NSteps - 1);
                  #local CurZ = TextDepth;
                  #local Theta = 0;
                  #for (J, 1, NSteps)
                     #local CurBorder = EffectWidth - EffectWidth * sin (radians (Theta));
                     MakeCharacter (I, CurBorder)
                     #local Segment = object {Character translate -1 * z scale <1, 1, SegmentLength>}
                     object {Segment translate CurZ * z}
                     #local CurZ = CurZ - SegmentLength;
                     #local Theta = Theta + DeltaTheta;
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -TextDepth * z //Back is at z = 0
            } //object
            #declare Character = object {TempCharacter}
         #break //#case
         #case (10) //Hollow hourglass (convex)
            #local EffectWidth = P1;
            #local WallThickness = P2 / TextScale_Final;
            #local NSteps = P3;
            #local TempCharacter = object {
               union {
                  #local SegmentLength = TextDepth / NSteps;
                  #local DeltaTheta = 180 / (NSteps - 1);
                  #local CurZ = TextDepth;
                  #local Theta = 0;
                  #for (J, 1, NSteps)
                     #local CurBorder = EffectWidth - EffectWidth * sin (radians (Theta));
                     MakeCharacter (I, CurBorder)
                     #local Character_Outer = object {Character}
                     MakeCharacter (I, CurBorder + WallThickness)
                     #local Character_Inner = object {Character scale <1, 1, 1.2>}
                     #local Segment = object {
                        difference {
                           object {Character_Outer}
                           object {Character_Inner translate -.1 * z}
                        } //difference
                        translate -1 * z
                        scale <1, 1, SegmentLength>
                     } //object
                     object {Segment translate CurZ * z}
                     #local CurZ = CurZ - SegmentLength;
                     #local Theta = Theta + DeltaTheta;
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -TextDepth * z //Back is at z = 0
            } //object
            #declare Character = object {TempCharacter}
         #break //#case
         #case (11) //Solid hourglass (concave)
            #local EffectWidth = P1;
            #local NSteps = P2;
            #local TempCharacter = object {
               union {
                  #local SegmentLength = TextDepth / NSteps;
                  #local DeltaTheta = 180 / (NSteps - 1);
                  #local CurZ = TextDepth;
                  #local Theta = 0;
                  #for (J, 1, NSteps)
                     #local CurBorder = EffectWidth * sin (radians (Theta));
                     MakeCharacter (I, CurBorder)
                     #local Segment = object {Character translate -1 * z scale <1, 1, SegmentLength>}
                     object {Segment translate CurZ * z}
                     #local CurZ = CurZ - SegmentLength;
                     #local Theta = Theta + DeltaTheta;
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -TextDepth * z //Back is at z = 0
            } //object
            #declare Character = object {TempCharacter}
         #break //#case
         #case (12) //Hollow hourglass (concave)
            #local EffectWidth = P1;
            #local WallThickness = P2 / TextScale_Final;
            #local NSteps = P3;
            #local TempCharacter = object {
               union {
                  #local SegmentLength = TextDepth / NSteps;
                  #local DeltaTheta = 180 / (NSteps - 1);
                  #local CurZ = TextDepth;
                  #local Theta = 0;
                  #for (J, 1, NSteps)
                     #local CurBorder = EffectWidth * sin (radians (Theta));
                     MakeCharacter (I, CurBorder)
                     #local Character_Outer = object {Character}
                     MakeCharacter (I, CurBorder + WallThickness)
                     #local Character_Inner = object {Character scale <1, 1, 1.2>}
                     #local Segment = object {
                        difference {
                           object {Character_Outer}
                           object {Character_Inner translate -.1 * z}
                        } //difference
                        translate -1 * z
                        scale <1, 1, SegmentLength>
                     } //object
                     object {Segment translate CurZ * z}
                     #local CurZ = CurZ - SegmentLength;
                     #local Theta = Theta + DeltaTheta;
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -TextDepth * z //Back is at z = 0
            } //object
            #declare Character = object {TempCharacter}
         #break //#case
         #case (13) //Solid hourglass (concave)
            #local EffectWidth = P1;
            #local NSteps = P2;
            #local TempCharacter = object {
               union {
                  #local SegmentLength = TextDepth / NSteps;
                  #local DeltaTheta = 90 / (NSteps - 1);
                  #local CurZ = TextDepth;
                  #local Theta = 0;
                  #for (J, 1, NSteps)
                     #local CurBorder = EffectWidth * sin (radians (Theta));
                     MakeCharacter (I, CurBorder)
                     #local Segment = object {Character translate -1 * z scale <1, 1, SegmentLength>}
                     object {Segment translate CurZ * z}
                     #local CurZ = CurZ - SegmentLength;
                     #local Theta = Theta + DeltaTheta;
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -TextDepth * z //Back is at z = 0
            } //object
            #declare Character = object {TempCharacter}
         #break //#case
         #case (14) //Hollow inside, bordered
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local SolidWidth = P3 / TextScale_Final;
            #local BorderDepth = P4 / 1.1;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local SolidWidth = SolidWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character texture {TextColor_Outer}}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Middle = object {Character texture {TextColor_Inner}}
            MakeCharacter (I, BorderWidth + SolidWidth + InvisibleBorderWidth)
            #local Character_Inner = object {Character texture {TextColor_Outer}}
            MakeCharacter (I, BorderWidth + SolidWidth + BorderWidth + InvisibleBorderWidth)
            #local Character_Hole = object {Character texture {TextColor_Outer}}
            #declare Character = object {
               difference {
                  object {Character_Outer}
                  object {Character_Middle scale <1, 1, 1.2> translate -.1 * z}
               } //difference
            } //object
            #declare Character = object {
               union {
                  object {Character}
                  object {Character_Middle translate BorderDepth * z}
               } //union
            } //object
            #declare Character = object {
               difference {
                  object {Character}
                  object {Character_Inner scale <1, 1, 1.2> translate -.1 * z}
               } //difference
            } //object
            #declare Character = object {
               union {
                  object {Character}
                  object {Character_Inner}
               } //union
            } //object
            #declare Character = object {
               difference {
                  object {Character}
                  object {Character_Hole scale <1, 1, 1.2> translate -.1 * z}
               } //difference
            } //object
            #declare Character = object {
               intersection {
                  object {Character scale <1, 1, 1.1>}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (15) //Hollow stripe with raised center
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local StripeWidth = P3 / TextScale_Final;
            #local StripeDepth = P4;
            #local InnerHeight = P5;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local StripeWidth = StripeWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #declare Character = object {Character scale <1, 1, StripeDepth + .1> texture {TextColor_Outer}}
            #local Character_Outer = object {
               difference {
                  object {Character_Outer}
                  object {Character translate -.1 * z}
               } //difference
            } //object
            MakeCharacter (I, BorderWidth + StripeWidth + InvisibleBorderWidth)
            #declare Character = object {
               intersection {
                  union {
                     object {Character_Outer}
                     object {Character translate -InnerHeight * z texture {TextColor_Inner}}
                  } //union
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (16) //Angled reverse bevel
            #local BorderDepth = P1;
            #local NSteps = P2;
            MakeCharacter (I, 0)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
            } //object
            #local Character_Inner = object {
               union {
                  #local Increment = BorderDepth / NSteps;
                  #for (J, 1, NSteps)
                     MakeCharacter (I, Increment * J)
                     #declare Character = object {
                        intersection {
                           object {Character translate .1 * z}
                           plane {z, 1}
                        } //intersection
                     } //object
                     object {Character translate (Increment * J) * z}
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -1 * z
            } //object
            #declare Character = object {
               difference {
                  object {Character_Outer}
                  object {Character_Inner}
               } //difference
               translate -TextDepth * z //Back is at z = 0
            } //object
         #break //#case
         #case (17) //Rounded reverse bevel/concave
            #local BorderDepth = P1;
            #local NSteps = P2;
            #local R = BorderDepth;
            MakeCharacter (I, 0)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
            } //object
            #local Character_Inner = object {
               union {
                  #local Increment = 90 / NSteps;
                  #for (J, 1, NSteps)
                     #local CurAngle = J * Increment;
                     #local CurBorderWidth = R * (1 + cos (radians (CurAngle + 180)));
                     #local CurBorderDepth = R * sin (radians (CurAngle));
                     MakeCharacter (I, CurBorderWidth)
                     #declare Character = object {
                        intersection {
                           object {Character translate .1 * z}
                           plane {z, 1}
                        } //intersection
                     } //object
                     object {Character translate CurBorderDepth * z}
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -1 * z
            } //object
            #declare Character = object {
               difference {
                  object {Character_Outer}
                  object {Character_Inner}
               } //difference
               translate -TextDepth * z //Back is at z = 0
            } //object
         #break //#case
         #case (18) //Rounded reverse bevel/convex
            #local BorderDepth = P1;
            #local NSteps = P2;
            #local R = BorderDepth;
            MakeCharacter (I, 0)
            #local Character_Outer = object {
               intersection {
                  object {Character scale <1, 1, 1.1> texture {TextColor_Outer}}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
            } //object
            #local Character_Inner = object {
               union {
                  #local Increment = 90 / NSteps;
                  #for (J, 1, NSteps)
                     #local CurAngle = J * Increment;
                     #local CurBorderDepth = R * (1 + cos (radians (CurAngle + 180)));
                     #local CurBorderWidth = R * sin (radians (CurAngle));
                     MakeCharacter (I, CurBorderWidth)
                     #declare Character = object {
                        intersection {
                           object {Character translate .1 * z}
                           plane {z, 1}
                        } //intersection
                     } //object
                     object {Character translate CurBorderDepth * z}
                  #end //#for
               } //union
               texture {TextColor_Outer}
               translate -1 * z
            } //object
            #declare Character = object {
               difference {
                  object {Character_Outer}
                  object {Character_Inner}
               } //difference
               translate -TextDepth * z //Back is at z = 0
            } //object
         #break //#case
         #case (19) //Stack
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local SpacerDepth = P3;
            #local NSteps = P4;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {Character scale <1, 1, 1.2> texture {TextColor_Outer}}
            MakeCharacter (I, InvisibleBorderWidth)
            #declare Character = object {Character texture {TextColor_Outer}}
            #declare Character = object {
               union {
                  difference {
                     intersection {
                        object {Character scale <1, 1, 1.1>}
                        plane {z, TextDepth texture {TextColor_Back}}
                     } //intersection
                     object {Character_Inner translate -.1 * z}
                  } //difference
                  #for (J, 1, NSteps - 1)
                     difference {
                        object {
                           Character
                           scale <1, 1, TextDepth>
                        } //object
                        object {Character_Inner translate -.1 * z}
                        translate (-TextDepth * J - SpacerDepth * J) * z
                     } //difference
                  #end //#for
               } //union
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (20) //Raised inner border
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local RaisedBorderWidth = P3 / TextScale_Final;
            #local RaisedBorderHeight = P4;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local RaisedBorderWidth = RaisedBorderWidth / S;
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character scale <1, 1, 1.1> texture {TextColor_Inner}}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {Character}
            MakeCharacter (I, BorderWidth + RaisedBorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {
               difference {
                  object {Character_Inner}
                  object {Character scale <1, 1, 1.2> translate -.1 * z}
               } //difference
               texture {TextColor_Outer}
            } //object
            #declare Character = object {
               intersection {
                  union {
                     object {Character_Outer}
                     object {Character_Inner translate -RaisedBorderHeight * z}
                  } //union
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (21) //Specular highlights
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local BorderWidth = P2 / TextScale_Final;
            #local BorderDepth = P3 / 1.1;
            #local Angle = P4;
            #local S = 1 / (1 - InvisibleBorderWidth * 2);
            #local BorderWidth = BorderWidth / S;
            #local R = BorderWidth * 2.5;
            #local XOffset = R * sin (radians (Angle));
            #local YOffset = R * cos (radians (Angle));
            MakeCharacter (I, InvisibleBorderWidth)
            #local Character_Outer = object {Character texture {TextColor_Outer}}
            MakeCharacter (I, BorderWidth + InvisibleBorderWidth)
            #local Character_Inner = object {
               intersection {
                  object {Character scale <1, 1, 1.1>}
                  plane {z, 1}
               } //intersection
               texture {TextColor_Inner}
            } //object
            #declare Character_Outer = object {
               difference {
                  object {Character_Outer}
                  object {Character_Inner translate (-1 + BorderDepth) * z}
               } //difference
            } //object
            MakeCharacter (I, BorderWidth * 2 + InvisibleBorderWidth)
            #local Character_Inner = object {Character}
            MakeCharacter (I, BorderWidth * 1 + InvisibleBorderWidth)
            #local Character_Inner = object {
               difference {
                  object {Character_Inner}
                  object {Character scale <1, 1, 1.2> translate <XOffset, YOffset, -.1>}
               } //difference
               texture {TextColor_Outer}
            } //object
            #declare Character = object {
               union {
                  object {Character_Outer}
                  object {Character_Inner}
               } //union
            } //object
            #declare Character = object {
               intersection {
                  object {Character scale <1, 1, 1.1>}
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, -TextDepth> //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
         #case (22) //Drop shadow
            #if (I = 81) //Heart
               #local InvisibleBorderWidth = 0;
            #else
               #local InvisibleBorderWidth = P1 / TextScale_Final;
            #end //#if
            #local Quadrant = P2; //1, 2, 3, or 4
            #local XYOffset_Solid = P3;
            #local XYOffset_Space = P4;
            #local LeaveInOffsetPosition = P5; //Set to "yes" if used as drop shadows, set to "no" if used as stand-alone characters.
            #local Angle = (Quadrant - 1) * 90 + 45;
            #switch (Quadrant)
               #case (1)
                  #local XSign = 1;
                  #local YSign = 1;
               #break //#case
               #case (2)
                  #local XSign = -1;
                  #local YSign = 1;
               #break //#case
               #case (3)
                  #local XSign = -1;
                  #local YSign = -1;
               #break //#case
               #case (4)
                  #local XSign = 1;
                  #local YSign = -1;
               #break //#case
            #end //#switch
            #local XOffset_Solid = XYOffset_Solid * XSign;
            #local YOffset_Solid = XYOffset_Solid * YSign;
            #local XOffset_Space = XYOffset_Space * XSign;
            #local YOffset_Space = XYOffset_Space * YSign;
            MakeCharacter (I, InvisibleBorderWidth)
            #declare Character = object {
               Character
               #local S = 1 / (1 - InvisibleBorderWidth * 2);
               scale <S, S, 1>
               translate <-InvisibleBorderWidth * S, -InvisibleBorderWidth * S, 0>
            } //object
            #declare Character = object {
               intersection {
                  difference {
                     object {Character scale <1, 1, 1.1> translate <XOffset_Solid, YOffset_Solid, 0>}
                     object {Character scale <1, 1, 1.2> translate -.1 * z}
                     texture {TextColor_Outer}
                  } //difference
                  plane {z, TextDepth texture {TextColor_Back}}
               } //intersection
               #if (LeaveInOffsetPosition)
                  translate <XOffset_Space, YOffset_Space, 0>
               #else
                  translate <-XOffset_Solid, -YOffset_Solid, 0>
               #end //#if
               translate -TextDepth * z //Back is at z = 0
               #declare BorderCharacterWidth [I] = BorderCharacterWidth [I] * S - InvisibleBorderWidth * S * 2;
            } //object
         #break //#case
      #end //#switch
      #declare BorderCharacter [I] = object {Character}
   #end //#for

#end //#macro Stylize
